# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import pipeline_list_params, pipeline_create_params, pipeline_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.pipeline import Pipeline
from ..types.pipeline_list_response import PipelineListResponse
from ..types.action_type_config_param import ActionTypeConfigParam
from ..types.pipeline_retrieve_response import PipelineRetrieveResponse

__all__ = ["PipelinesResource", "AsyncPipelinesResource"]


class PipelinesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PipelinesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#accessing-raw-response-data-eg-headers
        """
        return PipelinesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PipelinesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#with_streaming_response
        """
        return PipelinesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        output_schema: object,
        action_config: ActionTypeConfigParam | NotGiven = NOT_GIVEN,
        complex_tabular_transform_enabled: bool | NotGiven = NOT_GIVEN,
        independent_document_processing_enabled: bool | NotGiven = NOT_GIVEN,
        output_schema_name: str | NotGiven = NOT_GIVEN,
        webhook_enabled: bool | NotGiven = NOT_GIVEN,
        webhook_url: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Pipeline:
        """Creates a new pipeline to transform data, given an output schema.

        It returns the
        created pipeline's details. Pipelines are long-lived, so we recommend you create
        them outside of your application loop and reuse them.

        Args:
          name: Name of pipeline

          output_schema: Desired output structure defined in standard JSON Schema convention. Note - We
              DO NOT support non-alphanumeric characters in names of fields.

          action_config: Configuration of router that maps names of routes to respective pipelines to
              route to.

          complex_tabular_transform_enabled: Whether complex tabular transforms are enabled on the pipeline. This enables the
              pipeline to parse CSVs with multiple tables in the same file, and to transpose
              CSVs that can't be parsed row-wise.

          independent_document_processing_enabled: Whether independent transformations is enabled. For PDFs sent through the
              pipeline, this enables independent transformations for each individual page. For
              CSVs, this enables transforming chunks of rows in the CSV.

          output_schema_name: Name of output schema object.

          webhook_enabled: DEPRECATED - use subscriptions for webhook events. Whether webhook functionality
              is enabled.

          webhook_url: DEPRECATED - use subscriptions for webhook events. URL bem will send webhook
              requests to with successful transformation outputs if webhooks are enabled for
              the pipeline.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1-beta/pipelines",
            body=maybe_transform(
                {
                    "name": name,
                    "output_schema": output_schema,
                    "action_config": action_config,
                    "complex_tabular_transform_enabled": complex_tabular_transform_enabled,
                    "independent_document_processing_enabled": independent_document_processing_enabled,
                    "output_schema_name": output_schema_name,
                    "webhook_enabled": webhook_enabled,
                    "webhook_url": webhook_url,
                },
                pipeline_create_params.PipelineCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Pipeline,
        )

    def retrieve(
        self,
        pipeline_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PipelineRetrieveResponse:
        """
        Retrieves configuration of an existing pipeline.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not pipeline_id:
            raise ValueError(f"Expected a non-empty value for `pipeline_id` but received {pipeline_id!r}")
        return self._get(
            f"/v1-beta/pipelines/{pipeline_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PipelineRetrieveResponse,
        )

    def update(
        self,
        pipeline_id: str,
        *,
        action_config: ActionTypeConfigParam | NotGiven = NOT_GIVEN,
        complex_tabular_transform_enabled: bool | NotGiven = NOT_GIVEN,
        independent_document_processing_enabled: bool | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        output_schema: object | NotGiven = NOT_GIVEN,
        output_schema_name: str | NotGiven = NOT_GIVEN,
        webhook_enabled: bool | NotGiven = NOT_GIVEN,
        webhook_url: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Pipeline:
        """Updates an existing pipeline.

        Follow conventional PATCH behavior, so only
        included fields will be updated.

        Args:
          action_config: Configuration of router that maps names of routes to respective pipelines to
              route to.

          complex_tabular_transform_enabled: Whether complex tabular transforms are enabled on the pipeline. This enables the
              pipeline to parse CSVs with multiple tables in the same file, and to transpose
              CSVs that can't be parsed row-wise.

          independent_document_processing_enabled: Whether independent transformations is enabled. For PDFs sent through the
              pipeline, this enables independent transformations for each individual page. For
              CSVs, this enables transforming chunks of rows in the CSV.

          name: Name of pipeline

          output_schema: Desired output structure defined in standard JSON Schema convention. Note - We
              DO NOT support non-alphanumeric characters in names of fields.

          output_schema_name: Name of output schema object.

          webhook_enabled: DEPRECATED - use subscriptions for webhook events. Whether webhook functionality
              is enabled.

          webhook_url: DEPRECATED - use subscriptions for webhook events. URL bem will send webhook
              requests to with successful transformation outputs if webhooks are enabled for
              the pipeline.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not pipeline_id:
            raise ValueError(f"Expected a non-empty value for `pipeline_id` but received {pipeline_id!r}")
        return self._patch(
            f"/v1-beta/pipelines/{pipeline_id}",
            body=maybe_transform(
                {
                    "action_config": action_config,
                    "complex_tabular_transform_enabled": complex_tabular_transform_enabled,
                    "independent_document_processing_enabled": independent_document_processing_enabled,
                    "name": name,
                    "output_schema": output_schema,
                    "output_schema_name": output_schema_name,
                    "webhook_enabled": webhook_enabled,
                    "webhook_url": webhook_url,
                },
                pipeline_update_params.PipelineUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Pipeline,
        )

    def list(
        self,
        *,
        ending_before: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        starting_after: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PipelineListResponse:
        """
        Retrieves configurations for all existing pipelines.

        Args:
          ending_before: A cursor to use in pagination. `endingBefore` is a pipeline ID that defines your
              place in the list. For example, if you make a list request and receive 50
              objects, starting with `pl_2c9AXIj48cUYJtCuv1gsQtHGDzK`, your subsequent call
              can include `endingBefore=pl_2c9AXIj48cUYJtCuv1gsQtHGDzK` to fetch the previous
              page of the list.

          starting_after: A cursor to use in pagination. `startingAfter` is a pipeline ID that defines
              your place in the list. For example, if you make a list request and receive 50
              objects, ending with `pl_2c9AXIj48cUYJtCuv1gsQtHGDzK`, your subsequent call can
              include `startingAfter=pl_2c9AXIj48cUYJtCuv1gsQtHGDzK` to fetch the next page of
              the list.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1-beta/pipelines",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "limit": limit,
                        "starting_after": starting_after,
                    },
                    pipeline_list_params.PipelineListParams,
                ),
            ),
            cast_to=PipelineListResponse,
        )

    def delete(
        self,
        pipeline_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Deletes an existing pipeline and all related transformations.

        **IMPORTANT NOTE:** be sure you have exported any relevant transformations
        produced by the pipeline before deleting given they won't be accessible through
        our API after deleting the pipeline.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not pipeline_id:
            raise ValueError(f"Expected a non-empty value for `pipeline_id` but received {pipeline_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v1-beta/pipelines/{pipeline_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncPipelinesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPipelinesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPipelinesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPipelinesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#with_streaming_response
        """
        return AsyncPipelinesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        output_schema: object,
        action_config: ActionTypeConfigParam | NotGiven = NOT_GIVEN,
        complex_tabular_transform_enabled: bool | NotGiven = NOT_GIVEN,
        independent_document_processing_enabled: bool | NotGiven = NOT_GIVEN,
        output_schema_name: str | NotGiven = NOT_GIVEN,
        webhook_enabled: bool | NotGiven = NOT_GIVEN,
        webhook_url: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Pipeline:
        """Creates a new pipeline to transform data, given an output schema.

        It returns the
        created pipeline's details. Pipelines are long-lived, so we recommend you create
        them outside of your application loop and reuse them.

        Args:
          name: Name of pipeline

          output_schema: Desired output structure defined in standard JSON Schema convention. Note - We
              DO NOT support non-alphanumeric characters in names of fields.

          action_config: Configuration of router that maps names of routes to respective pipelines to
              route to.

          complex_tabular_transform_enabled: Whether complex tabular transforms are enabled on the pipeline. This enables the
              pipeline to parse CSVs with multiple tables in the same file, and to transpose
              CSVs that can't be parsed row-wise.

          independent_document_processing_enabled: Whether independent transformations is enabled. For PDFs sent through the
              pipeline, this enables independent transformations for each individual page. For
              CSVs, this enables transforming chunks of rows in the CSV.

          output_schema_name: Name of output schema object.

          webhook_enabled: DEPRECATED - use subscriptions for webhook events. Whether webhook functionality
              is enabled.

          webhook_url: DEPRECATED - use subscriptions for webhook events. URL bem will send webhook
              requests to with successful transformation outputs if webhooks are enabled for
              the pipeline.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1-beta/pipelines",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "output_schema": output_schema,
                    "action_config": action_config,
                    "complex_tabular_transform_enabled": complex_tabular_transform_enabled,
                    "independent_document_processing_enabled": independent_document_processing_enabled,
                    "output_schema_name": output_schema_name,
                    "webhook_enabled": webhook_enabled,
                    "webhook_url": webhook_url,
                },
                pipeline_create_params.PipelineCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Pipeline,
        )

    async def retrieve(
        self,
        pipeline_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PipelineRetrieveResponse:
        """
        Retrieves configuration of an existing pipeline.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not pipeline_id:
            raise ValueError(f"Expected a non-empty value for `pipeline_id` but received {pipeline_id!r}")
        return await self._get(
            f"/v1-beta/pipelines/{pipeline_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PipelineRetrieveResponse,
        )

    async def update(
        self,
        pipeline_id: str,
        *,
        action_config: ActionTypeConfigParam | NotGiven = NOT_GIVEN,
        complex_tabular_transform_enabled: bool | NotGiven = NOT_GIVEN,
        independent_document_processing_enabled: bool | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        output_schema: object | NotGiven = NOT_GIVEN,
        output_schema_name: str | NotGiven = NOT_GIVEN,
        webhook_enabled: bool | NotGiven = NOT_GIVEN,
        webhook_url: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Pipeline:
        """Updates an existing pipeline.

        Follow conventional PATCH behavior, so only
        included fields will be updated.

        Args:
          action_config: Configuration of router that maps names of routes to respective pipelines to
              route to.

          complex_tabular_transform_enabled: Whether complex tabular transforms are enabled on the pipeline. This enables the
              pipeline to parse CSVs with multiple tables in the same file, and to transpose
              CSVs that can't be parsed row-wise.

          independent_document_processing_enabled: Whether independent transformations is enabled. For PDFs sent through the
              pipeline, this enables independent transformations for each individual page. For
              CSVs, this enables transforming chunks of rows in the CSV.

          name: Name of pipeline

          output_schema: Desired output structure defined in standard JSON Schema convention. Note - We
              DO NOT support non-alphanumeric characters in names of fields.

          output_schema_name: Name of output schema object.

          webhook_enabled: DEPRECATED - use subscriptions for webhook events. Whether webhook functionality
              is enabled.

          webhook_url: DEPRECATED - use subscriptions for webhook events. URL bem will send webhook
              requests to with successful transformation outputs if webhooks are enabled for
              the pipeline.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not pipeline_id:
            raise ValueError(f"Expected a non-empty value for `pipeline_id` but received {pipeline_id!r}")
        return await self._patch(
            f"/v1-beta/pipelines/{pipeline_id}",
            body=await async_maybe_transform(
                {
                    "action_config": action_config,
                    "complex_tabular_transform_enabled": complex_tabular_transform_enabled,
                    "independent_document_processing_enabled": independent_document_processing_enabled,
                    "name": name,
                    "output_schema": output_schema,
                    "output_schema_name": output_schema_name,
                    "webhook_enabled": webhook_enabled,
                    "webhook_url": webhook_url,
                },
                pipeline_update_params.PipelineUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Pipeline,
        )

    async def list(
        self,
        *,
        ending_before: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        starting_after: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PipelineListResponse:
        """
        Retrieves configurations for all existing pipelines.

        Args:
          ending_before: A cursor to use in pagination. `endingBefore` is a pipeline ID that defines your
              place in the list. For example, if you make a list request and receive 50
              objects, starting with `pl_2c9AXIj48cUYJtCuv1gsQtHGDzK`, your subsequent call
              can include `endingBefore=pl_2c9AXIj48cUYJtCuv1gsQtHGDzK` to fetch the previous
              page of the list.

          starting_after: A cursor to use in pagination. `startingAfter` is a pipeline ID that defines
              your place in the list. For example, if you make a list request and receive 50
              objects, ending with `pl_2c9AXIj48cUYJtCuv1gsQtHGDzK`, your subsequent call can
              include `startingAfter=pl_2c9AXIj48cUYJtCuv1gsQtHGDzK` to fetch the next page of
              the list.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1-beta/pipelines",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "ending_before": ending_before,
                        "limit": limit,
                        "starting_after": starting_after,
                    },
                    pipeline_list_params.PipelineListParams,
                ),
            ),
            cast_to=PipelineListResponse,
        )

    async def delete(
        self,
        pipeline_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Deletes an existing pipeline and all related transformations.

        **IMPORTANT NOTE:** be sure you have exported any relevant transformations
        produced by the pipeline before deleting given they won't be accessible through
        our API after deleting the pipeline.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not pipeline_id:
            raise ValueError(f"Expected a non-empty value for `pipeline_id` but received {pipeline_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v1-beta/pipelines/{pipeline_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class PipelinesResourceWithRawResponse:
    def __init__(self, pipelines: PipelinesResource) -> None:
        self._pipelines = pipelines

        self.create = to_raw_response_wrapper(
            pipelines.create,
        )
        self.retrieve = to_raw_response_wrapper(
            pipelines.retrieve,
        )
        self.update = to_raw_response_wrapper(
            pipelines.update,
        )
        self.list = to_raw_response_wrapper(
            pipelines.list,
        )
        self.delete = to_raw_response_wrapper(
            pipelines.delete,
        )


class AsyncPipelinesResourceWithRawResponse:
    def __init__(self, pipelines: AsyncPipelinesResource) -> None:
        self._pipelines = pipelines

        self.create = async_to_raw_response_wrapper(
            pipelines.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            pipelines.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            pipelines.update,
        )
        self.list = async_to_raw_response_wrapper(
            pipelines.list,
        )
        self.delete = async_to_raw_response_wrapper(
            pipelines.delete,
        )


class PipelinesResourceWithStreamingResponse:
    def __init__(self, pipelines: PipelinesResource) -> None:
        self._pipelines = pipelines

        self.create = to_streamed_response_wrapper(
            pipelines.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            pipelines.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            pipelines.update,
        )
        self.list = to_streamed_response_wrapper(
            pipelines.list,
        )
        self.delete = to_streamed_response_wrapper(
            pipelines.delete,
        )


class AsyncPipelinesResourceWithStreamingResponse:
    def __init__(self, pipelines: AsyncPipelinesResource) -> None:
        self._pipelines = pipelines

        self.create = async_to_streamed_response_wrapper(
            pipelines.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            pipelines.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            pipelines.update,
        )
        self.list = async_to_streamed_response_wrapper(
            pipelines.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            pipelines.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.webhook_secret import WebhookSecret

__all__ = ["WebhookSecretResource", "AsyncWebhookSecretResource"]


class WebhookSecretResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> WebhookSecretResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#accessing-raw-response-data-eg-headers
        """
        return WebhookSecretResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WebhookSecretResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#with_streaming_response
        """
        return WebhookSecretResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WebhookSecret:
        """Generates a new webhook secret to be used for webhook signatures.

        If a webhook
        secret already exists, this endpoint will overwrite the previous secret and
        generate a new one.
        """
        return self._post(
            "/v1-beta/webhook-secret",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookSecret,
        )

    def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WebhookSecret:
        """Gets the current webhook secret for your account."""
        return self._get(
            "/v1-beta/webhook-secret",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookSecret,
        )


class AsyncWebhookSecretResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncWebhookSecretResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWebhookSecretResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWebhookSecretResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#with_streaming_response
        """
        return AsyncWebhookSecretResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WebhookSecret:
        """Generates a new webhook secret to be used for webhook signatures.

        If a webhook
        secret already exists, this endpoint will overwrite the previous secret and
        generate a new one.
        """
        return await self._post(
            "/v1-beta/webhook-secret",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookSecret,
        )

    async def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WebhookSecret:
        """Gets the current webhook secret for your account."""
        return await self._get(
            "/v1-beta/webhook-secret",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookSecret,
        )


class WebhookSecretResourceWithRawResponse:
    def __init__(self, webhook_secret: WebhookSecretResource) -> None:
        self._webhook_secret = webhook_secret

        self.create = to_raw_response_wrapper(
            webhook_secret.create,
        )
        self.retrieve = to_raw_response_wrapper(
            webhook_secret.retrieve,
        )


class AsyncWebhookSecretResourceWithRawResponse:
    def __init__(self, webhook_secret: AsyncWebhookSecretResource) -> None:
        self._webhook_secret = webhook_secret

        self.create = async_to_raw_response_wrapper(
            webhook_secret.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            webhook_secret.retrieve,
        )


class WebhookSecretResourceWithStreamingResponse:
    def __init__(self, webhook_secret: WebhookSecretResource) -> None:
        self._webhook_secret = webhook_secret

        self.create = to_streamed_response_wrapper(
            webhook_secret.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            webhook_secret.retrieve,
        )


class AsyncWebhookSecretResourceWithStreamingResponse:
    def __init__(self, webhook_secret: AsyncWebhookSecretResource) -> None:
        self._webhook_secret = webhook_secret

        self.create = async_to_streamed_response_wrapper(
            webhook_secret.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            webhook_secret.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["SplitConfigSemanticPageItemClassParam"]


class SplitConfigSemanticPageItemClassParam(TypedDict, total=False):
    name: Required[str]

    description: str

    next_action_type_config_id: Annotated[str, PropertyInfo(alias="nextActionTypeConfigID")]
    """The unique ID of the action type configuration you want to use for this action."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .task import Task
from .._models import BaseModel

__all__ = ["TaskListResponse"]


class TaskListResponse(BaseModel):
    tasks: Optional[List[Task]] = None

    total_count: Optional[int] = FieldInfo(alias="totalCount", default=None)
    """The total number of results available."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["EmailActionBase"]


class EmailActionBase(BaseModel):
    template_variables: object = FieldInfo(alias="templateVariables")
    """Template variables to be used in the email body and subject.

    Templates are injected into the email body and subject as
    `{{template_variable}}`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .task_status import TaskStatus

__all__ = ["Task"]


class Task(BaseModel):
    status: TaskStatus
    """The status of the task."""

    task_id: str = FieldInfo(alias="taskID")
    """Unique ID generated by bem following Segment's KSUID conventions."""

    type: Literal["transformation"]
    """The type of the task."""

    created_at: Optional[datetime] = FieldInfo(alias="createdAt", default=None)
    """Timestamp when task was initiated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .action_type import ActionType

__all__ = ["ActionCreateBaseParam"]


class ActionCreateBaseParam(TypedDict, total=False):
    action_type: Required[Annotated[ActionType, PropertyInfo(alias="actionType")]]
    """The type of the action."""

    action_type_config_id: Required[Annotated[str, PropertyInfo(alias="actionTypeConfigID")]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from bem import BemSDK, AsyncBemSDK
from bem.types import (
    Pipeline,
    PipelineListResponse,
    PipelineRetrieveResponse,
)
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPipelines:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: BemSDK) -> None:
        pipeline = client.pipelines.create(
            name="Freight Load Pipeline",
            output_schema={
                "value": {
                    "type": "object",
                    "required": ["tenders"],
                    "properties": {
                        "tenders": {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "required": [
                                    "loadReference",
                                    "origin",
                                    "destination",
                                    "weightTons",
                                    "loadType",
                                    "desiredDeliveryDate",
                                    "bidSubmissionDeadline",
                                    "submitter",
                                ],
                                "properties": {
                                    "origin": {
                                        "type": "string",
                                        "description": "The starting point of the shipment.",
                                    },
                                    "loadType": {
                                        "type": "string",
                                        "description": "The type of goods being shipped.",
                                    },
                                    "submitter": {
                                        "type": "object",
                                        "required": ["name", "position", "contactInfo"],
                                        "properties": {
                                            "name": {
                                                "type": "string",
                                                "description": "Name of the person submitting the tender.",
                                            },
                                            "position": {
                                                "type": "string",
                                                "description": "Position of the submitter within their company.",
                                            },
                                            "contactInfo": {
                                                "type": "object",
                                                "required": ["email"],
                                                "properties": {
                                                    "email": {
                                                        "type": "string",
                                                        "format": "email",
                                                        "description": "Email address of the submitter.",
                                                    },
                                                    "phone": {
                                                        "type": "string",
                                                        "description": "Phone number of the submitter.",
                                                    },
                                                },
                                            },
                                        },
                                    },
                                    "weightTons": {
                                        "type": "number",
                                        "description": "The weight of the load in tons.",
                                    },
                                    "destination": {
                                        "type": "string",
                                        "description": "The endpoint of the shipment.",
                                    },
                                    "loadReference": {
                                        "type": "string",
                                        "description": "Unique identifier for the load tender.",
                                    },
                                    "desiredDeliveryDate": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The preferred date for the shipment to be delivered.",
                                    },
                                    "bidSubmissionDeadline": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The deadline for submitting bids.",
                                    },
                                },
                            },
                        }
                    },
                }
            },
        )
        assert_matches_type(Pipeline, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: BemSDK) -> None:
        pipeline = client.pipelines.create(
            name="Freight Load Pipeline",
            output_schema={
                "value": {
                    "type": "object",
                    "required": ["tenders"],
                    "properties": {
                        "tenders": {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "required": [
                                    "loadReference",
                                    "origin",
                                    "destination",
                                    "weightTons",
                                    "loadType",
                                    "desiredDeliveryDate",
                                    "bidSubmissionDeadline",
                                    "submitter",
                                ],
                                "properties": {
                                    "origin": {
                                        "type": "string",
                                        "description": "The starting point of the shipment.",
                                    },
                                    "loadType": {
                                        "type": "string",
                                        "description": "The type of goods being shipped.",
                                    },
                                    "submitter": {
                                        "type": "object",
                                        "required": ["name", "position", "contactInfo"],
                                        "properties": {
                                            "name": {
                                                "type": "string",
                                                "description": "Name of the person submitting the tender.",
                                            },
                                            "position": {
                                                "type": "string",
                                                "description": "Position of the submitter within their company.",
                                            },
                                            "contactInfo": {
                                                "type": "object",
                                                "required": ["email"],
                                                "properties": {
                                                    "email": {
                                                        "type": "string",
                                                        "format": "email",
                                                        "description": "Email address of the submitter.",
                                                    },
                                                    "phone": {
                                                        "type": "string",
                                                        "description": "Phone number of the submitter.",
                                                    },
                                                },
                                            },
                                        },
                                    },
                                    "weightTons": {
                                        "type": "number",
                                        "description": "The weight of the load in tons.",
                                    },
                                    "destination": {
                                        "type": "string",
                                        "description": "The endpoint of the shipment.",
                                    },
                                    "loadReference": {
                                        "type": "string",
                                        "description": "Unique identifier for the load tender.",
                                    },
                                    "desiredDeliveryDate": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The preferred date for the shipment to be delivered.",
                                    },
                                    "bidSubmissionDeadline": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The deadline for submitting bids.",
                                    },
                                },
                            },
                        }
                    },
                }
            },
            action_config={
                "action_type_config_id": "actionTypeConfigID",
                "name": "name",
                "action_type": "transform",
                "complex_tabular_transform_enabled": True,
                "email_address": "eml_2c9AXFXHwiaL4vPXDTOS171OJ8T@pipeline.bem.ai",
                "independent_document_processing_enabled": False,
                "output_schema": {
                    "value": {
                        "type": "object",
                        "required": ["tenders"],
                        "properties": {
                            "tenders": {
                                "type": "array",
                                "items": {
                                    "type": "object",
                                    "required": [
                                        "loadReference",
                                        "origin",
                                        "destination",
                                        "weightTons",
                                        "loadType",
                                        "desiredDeliveryDate",
                                        "bidSubmissionDeadline",
                                        "submitter",
                                    ],
                                    "properties": {
                                        "origin": {
                                            "type": "string",
                                            "description": "The starting point of the shipment.",
                                        },
                                        "loadType": {
                                            "type": "string",
                                            "description": "The type of goods being shipped.",
                                        },
                                        "submitter": {
                                            "type": "object",
                                            "required": ["name", "position", "contactInfo"],
                                            "properties": {
                                                "name": {
                                                    "type": "string",
                                                    "description": "Name of the person submitting the tender.",
                                                },
                                                "position": {
                                                    "type": "string",
                                                    "description": "Position of the submitter within their company.",
                                                },
                                                "contactInfo": {
                                                    "type": "object",
                                                    "required": ["email"],
                                                    "properties": {
                                                        "email": {
                                                            "type": "string",
                                                            "format": "email",
                                                            "description": "Email address of the submitter.",
                                                        },
                                                        "phone": {
                                                            "type": "string",
                                                            "description": "Phone number of the submitter.",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                        "weightTons": {
                                            "type": "number",
                                            "description": "The weight of the load in tons.",
                                        },
                                        "destination": {
                                            "type": "string",
                                            "description": "The endpoint of the shipment.",
                                        },
                                        "loadReference": {
                                            "type": "string",
                                            "description": "Unique identifier for the load tender.",
                                        },
                                        "desiredDeliveryDate": {
                                            "type": "string",
                                            "format": "date",
                                            "description": "The preferred date for the shipment to be delivered.",
                                        },
                                        "bidSubmissionDeadline": {
                                            "type": "string",
                                            "format": "date",
                                            "description": "The deadline for submitting bids.",
                                        },
                                    },
                                },
                            }
                        },
                    }
                },
                "output_schema_name": "Freight Load Schema",
                "next_action_type_config_id": "nextActionTypeConfigID",
            },
            complex_tabular_transform_enabled=False,
            independent_document_processing_enabled=False,
            output_schema_name="Freight Load Schema",
            webhook_enabled=True,
            webhook_url="https://bem-example.ai/test/url",
        )
        assert_matches_type(Pipeline, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: BemSDK) -> None:
        response = client.pipelines.with_raw_response.create(
            name="Freight Load Pipeline",
            output_schema={
                "value": {
                    "type": "object",
                    "required": ["tenders"],
                    "properties": {
                        "tenders": {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "required": [
                                    "loadReference",
                                    "origin",
                                    "destination",
                                    "weightTons",
                                    "loadType",
                                    "desiredDeliveryDate",
                                    "bidSubmissionDeadline",
                                    "submitter",
                                ],
                                "properties": {
                                    "origin": {
                                        "type": "string",
                                        "description": "The starting point of the shipment.",
                                    },
                                    "loadType": {
                                        "type": "string",
                                        "description": "The type of goods being shipped.",
                                    },
                                    "submitter": {
                                        "type": "object",
                                        "required": ["name", "position", "contactInfo"],
                                        "properties": {
                                            "name": {
                                                "type": "string",
                                                "description": "Name of the person submitting the tender.",
                                            },
                                            "position": {
                                                "type": "string",
                                                "description": "Position of the submitter within their company.",
                                            },
                                            "contactInfo": {
                                                "type": "object",
                                                "required": ["email"],
                                                "properties": {
                                                    "email": {
                                                        "type": "string",
                                                        "format": "email",
                                                        "description": "Email address of the submitter.",
                                                    },
                                                    "phone": {
                                                        "type": "string",
                                                        "description": "Phone number of the submitter.",
                                                    },
                                                },
                                            },
                                        },
                                    },
                                    "weightTons": {
                                        "type": "number",
                                        "description": "The weight of the load in tons.",
                                    },
                                    "destination": {
                                        "type": "string",
                                        "description": "The endpoint of the shipment.",
                                    },
                                    "loadReference": {
                                        "type": "string",
                                        "description": "Unique identifier for the load tender.",
                                    },
                                    "desiredDeliveryDate": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The preferred date for the shipment to be delivered.",
                                    },
                                    "bidSubmissionDeadline": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The deadline for submitting bids.",
                                    },
                                },
                            },
                        }
                    },
                }
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pipeline = response.parse()
        assert_matches_type(Pipeline, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: BemSDK) -> None:
        with client.pipelines.with_streaming_response.create(
            name="Freight Load Pipeline",
            output_schema={
                "value": {
                    "type": "object",
                    "required": ["tenders"],
                    "properties": {
                        "tenders": {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "required": [
                                    "loadReference",
                                    "origin",
                                    "destination",
                                    "weightTons",
                                    "loadType",
                                    "desiredDeliveryDate",
                                    "bidSubmissionDeadline",
                                    "submitter",
                                ],
                                "properties": {
                                    "origin": {
                                        "type": "string",
                                        "description": "The starting point of the shipment.",
                                    },
                                    "loadType": {
                                        "type": "string",
                                        "description": "The type of goods being shipped.",
                                    },
                                    "submitter": {
                                        "type": "object",
                                        "required": ["name", "position", "contactInfo"],
                                        "properties": {
                                            "name": {
                                                "type": "string",
                                                "description": "Name of the person submitting the tender.",
                                            },
                                            "position": {
                                                "type": "string",
                                                "description": "Position of the submitter within their company.",
                                            },
                                            "contactInfo": {
                                                "type": "object",
                                                "required": ["email"],
                                                "properties": {
                                                    "email": {
                                                        "type": "string",
                                                        "format": "email",
                                                        "description": "Email address of the submitter.",
                                                    },
                                                    "phone": {
                                                        "type": "string",
                                                        "description": "Phone number of the submitter.",
                                                    },
                                                },
                                            },
                                        },
                                    },
                                    "weightTons": {
                                        "type": "number",
                                        "description": "The weight of the load in tons.",
                                    },
                                    "destination": {
                                        "type": "string",
                                        "description": "The endpoint of the shipment.",
                                    },
                                    "loadReference": {
                                        "type": "string",
                                        "description": "Unique identifier for the load tender.",
                                    },
                                    "desiredDeliveryDate": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The preferred date for the shipment to be delivered.",
                                    },
                                    "bidSubmissionDeadline": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The deadline for submitting bids.",
                                    },
                                },
                            },
                        }
                    },
                }
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pipeline = response.parse()
            assert_matches_type(Pipeline, pipeline, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: BemSDK) -> None:
        pipeline = client.pipelines.retrieve(
            "pipelineID",
        )
        assert_matches_type(PipelineRetrieveResponse, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: BemSDK) -> None:
        response = client.pipelines.with_raw_response.retrieve(
            "pipelineID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pipeline = response.parse()
        assert_matches_type(PipelineRetrieveResponse, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: BemSDK) -> None:
        with client.pipelines.with_streaming_response.retrieve(
            "pipelineID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pipeline = response.parse()
            assert_matches_type(PipelineRetrieveResponse, pipeline, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: BemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pipeline_id` but received ''"):
            client.pipelines.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: BemSDK) -> None:
        pipeline = client.pipelines.update(
            pipeline_id="pipelineID",
        )
        assert_matches_type(Pipeline, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: BemSDK) -> None:
        pipeline = client.pipelines.update(
            pipeline_id="pipelineID",
            action_config={
                "action_type_config_id": "actionTypeConfigID",
                "name": "name",
                "action_type": "transform",
                "complex_tabular_transform_enabled": True,
                "email_address": "eml_2c9AXFXHwiaL4vPXDTOS171OJ8T@pipeline.bem.ai",
                "independent_document_processing_enabled": False,
                "output_schema": {
                    "value": {
                        "type": "object",
                        "required": ["tenders"],
                        "properties": {
                            "tenders": {
                                "type": "array",
                                "items": {
                                    "type": "object",
                                    "required": [
                                        "loadReference",
                                        "origin",
                                        "destination",
                                        "weightTons",
                                        "loadType",
                                        "desiredDeliveryDate",
                                        "bidSubmissionDeadline",
                                        "submitter",
                                    ],
                                    "properties": {
                                        "origin": {
                                            "type": "string",
                                            "description": "The starting point of the shipment.",
                                        },
                                        "loadType": {
                                            "type": "string",
                                            "description": "The type of goods being shipped.",
                                        },
                                        "submitter": {
                                            "type": "object",
                                            "required": ["name", "position", "contactInfo"],
                                            "properties": {
                                                "name": {
                                                    "type": "string",
                                                    "description": "Name of the person submitting the tender.",
                                                },
                                                "position": {
                                                    "type": "string",
                                                    "description": "Position of the submitter within their company.",
                                                },
                                                "contactInfo": {
                                                    "type": "object",
                                                    "required": ["email"],
                                                    "properties": {
                                                        "email": {
                                                            "type": "string",
                                                            "format": "email",
                                                            "description": "Email address of the submitter.",
                                                        },
                                                        "phone": {
                                                            "type": "string",
                                                            "description": "Phone number of the submitter.",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                        "weightTons": {
                                            "type": "number",
                                            "description": "The weight of the load in tons.",
                                        },
                                        "destination": {
                                            "type": "string",
                                            "description": "The endpoint of the shipment.",
                                        },
                                        "loadReference": {
                                            "type": "string",
                                            "description": "Unique identifier for the load tender.",
                                        },
                                        "desiredDeliveryDate": {
                                            "type": "string",
                                            "format": "date",
                                            "description": "The preferred date for the shipment to be delivered.",
                                        },
                                        "bidSubmissionDeadline": {
                                            "type": "string",
                                            "format": "date",
                                            "description": "The deadline for submitting bids.",
                                        },
                                    },
                                },
                            }
                        },
                    }
                },
                "output_schema_name": "Freight Load Schema",
                "next_action_type_config_id": "nextActionTypeConfigID",
            },
            complex_tabular_transform_enabled=False,
            independent_document_processing_enabled=False,
            name="Freight invoices pipeline",
            output_schema={
                "value": {
                    "type": "object",
                    "required": ["tenders"],
                    "properties": {
                        "tenders": {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "required": [
                                    "loadReference",
                                    "origin",
                                    "destination",
                                    "weightTons",
                                    "loadType",
                                    "desiredDeliveryDate",
                                    "bidSubmissionDeadline",
                                    "submitter",
                                ],
                                "properties": {
                                    "origin": {
                                        "type": "string",
                                        "description": "The starting point of the shipment.",
                                    },
                                    "loadType": {
                                        "type": "string",
                                        "description": "The type of goods being shipped.",
                                    },
                                    "submitter": {
                                        "type": "object",
                                        "required": ["name", "position", "contactInfo"],
                                        "properties": {
                                            "name": {
                                                "type": "string",
                                                "description": "Name of the person submitting the tender.",
                                            },
                                            "position": {
                                                "type": "string",
                                                "description": "Position of the submitter within their company.",
                                            },
                                            "contactInfo": {
                                                "type": "object",
                                                "required": ["email"],
                                                "properties": {
                                                    "email": {
                                                        "type": "string",
                                                        "format": "email",
                                                        "description": "Email address of the submitter.",
                                                    },
                                                    "phone": {
                                                        "type": "string",
                                                        "description": "Phone number of the submitter.",
                                                    },
                                                },
                                            },
                                        },
                                    },
                                    "weightTons": {
                                        "type": "number",
                                        "description": "The weight of the load in tons.",
                                    },
                                    "destination": {
                                        "type": "string",
                                        "description": "The endpoint of the shipment.",
                                    },
                                    "loadReference": {
                                        "type": "string",
                                        "description": "Unique identifier for the load tender.",
                                    },
                                    "desiredDeliveryDate": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The preferred date for the shipment to be delivered.",
                                    },
                                    "bidSubmissionDeadline": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The deadline for submitting bids.",
                                    },
                                },
                            },
                        }
                    },
                }
            },
            output_schema_name="Freight invoices schema",
            webhook_enabled=True,
            webhook_url="https://bem-example.ai/test/url",
        )
        assert_matches_type(Pipeline, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: BemSDK) -> None:
        response = client.pipelines.with_raw_response.update(
            pipeline_id="pipelineID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pipeline = response.parse()
        assert_matches_type(Pipeline, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: BemSDK) -> None:
        with client.pipelines.with_streaming_response.update(
            pipeline_id="pipelineID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pipeline = response.parse()
            assert_matches_type(Pipeline, pipeline, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update(self, client: BemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pipeline_id` but received ''"):
            client.pipelines.with_raw_response.update(
                pipeline_id="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: BemSDK) -> None:
        pipeline = client.pipelines.list()
        assert_matches_type(PipelineListResponse, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_with_all_params(self, client: BemSDK) -> None:
        pipeline = client.pipelines.list(
            ending_before="endingBefore",
            limit=1,
            starting_after="startingAfter",
        )
        assert_matches_type(PipelineListResponse, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: BemSDK) -> None:
        response = client.pipelines.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pipeline = response.parse()
        assert_matches_type(PipelineListResponse, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: BemSDK) -> None:
        with client.pipelines.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pipeline = response.parse()
            assert_matches_type(PipelineListResponse, pipeline, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: BemSDK) -> None:
        pipeline = client.pipelines.delete(
            "pipelineID",
        )
        assert pipeline is None

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: BemSDK) -> None:
        response = client.pipelines.with_raw_response.delete(
            "pipelineID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pipeline = response.parse()
        assert pipeline is None

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: BemSDK) -> None:
        with client.pipelines.with_streaming_response.delete(
            "pipelineID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pipeline = response.parse()
            assert pipeline is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_delete(self, client: BemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pipeline_id` but received ''"):
            client.pipelines.with_raw_response.delete(
                "",
            )


class TestAsyncPipelines:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncBemSDK) -> None:
        pipeline = await async_client.pipelines.create(
            name="Freight Load Pipeline",
            output_schema={
                "value": {
                    "type": "object",
                    "required": ["tenders"],
                    "properties": {
                        "tenders": {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "required": [
                                    "loadReference",
                                    "origin",
                                    "destination",
                                    "weightTons",
                                    "loadType",
                                    "desiredDeliveryDate",
                                    "bidSubmissionDeadline",
                                    "submitter",
                                ],
                                "properties": {
                                    "origin": {
                                        "type": "string",
                                        "description": "The starting point of the shipment.",
                                    },
                                    "loadType": {
                                        "type": "string",
                                        "description": "The type of goods being shipped.",
                                    },
                                    "submitter": {
                                        "type": "object",
                                        "required": ["name", "position", "contactInfo"],
                                        "properties": {
                                            "name": {
                                                "type": "string",
                                                "description": "Name of the person submitting the tender.",
                                            },
                                            "position": {
                                                "type": "string",
                                                "description": "Position of the submitter within their company.",
                                            },
                                            "contactInfo": {
                                                "type": "object",
                                                "required": ["email"],
                                                "properties": {
                                                    "email": {
                                                        "type": "string",
                                                        "format": "email",
                                                        "description": "Email address of the submitter.",
                                                    },
                                                    "phone": {
                                                        "type": "string",
                                                        "description": "Phone number of the submitter.",
                                                    },
                                                },
                                            },
                                        },
                                    },
                                    "weightTons": {
                                        "type": "number",
                                        "description": "The weight of the load in tons.",
                                    },
                                    "destination": {
                                        "type": "string",
                                        "description": "The endpoint of the shipment.",
                                    },
                                    "loadReference": {
                                        "type": "string",
                                        "description": "Unique identifier for the load tender.",
                                    },
                                    "desiredDeliveryDate": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The preferred date for the shipment to be delivered.",
                                    },
                                    "bidSubmissionDeadline": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The deadline for submitting bids.",
                                    },
                                },
                            },
                        }
                    },
                }
            },
        )
        assert_matches_type(Pipeline, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncBemSDK) -> None:
        pipeline = await async_client.pipelines.create(
            name="Freight Load Pipeline",
            output_schema={
                "value": {
                    "type": "object",
                    "required": ["tenders"],
                    "properties": {
                        "tenders": {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "required": [
                                    "loadReference",
                                    "origin",
                                    "destination",
                                    "weightTons",
                                    "loadType",
                                    "desiredDeliveryDate",
                                    "bidSubmissionDeadline",
                                    "submitter",
                                ],
                                "properties": {
                                    "origin": {
                                        "type": "string",
                                        "description": "The starting point of the shipment.",
                                    },
                                    "loadType": {
                                        "type": "string",
                                        "description": "The type of goods being shipped.",
                                    },
                                    "submitter": {
                                        "type": "object",
                                        "required": ["name", "position", "contactInfo"],
                                        "properties": {
                                            "name": {
                                                "type": "string",
                                                "description": "Name of the person submitting the tender.",
                                            },
                                            "position": {
                                                "type": "string",
                                                "description": "Position of the submitter within their company.",
                                            },
                                            "contactInfo": {
                                                "type": "object",
                                                "required": ["email"],
                                                "properties": {
                                                    "email": {
                                                        "type": "string",
                                                        "format": "email",
                                                        "description": "Email address of the submitter.",
                                                    },
                                                    "phone": {
                                                        "type": "string",
                                                        "description": "Phone number of the submitter.",
                                                    },
                                                },
                                            },
                                        },
                                    },
                                    "weightTons": {
                                        "type": "number",
                                        "description": "The weight of the load in tons.",
                                    },
                                    "destination": {
                                        "type": "string",
                                        "description": "The endpoint of the shipment.",
                                    },
                                    "loadReference": {
                                        "type": "string",
                                        "description": "Unique identifier for the load tender.",
                                    },
                                    "desiredDeliveryDate": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The preferred date for the shipment to be delivered.",
                                    },
                                    "bidSubmissionDeadline": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The deadline for submitting bids.",
                                    },
                                },
                            },
                        }
                    },
                }
            },
            action_config={
                "action_type_config_id": "actionTypeConfigID",
                "name": "name",
                "action_type": "transform",
                "complex_tabular_transform_enabled": True,
                "email_address": "eml_2c9AXFXHwiaL4vPXDTOS171OJ8T@pipeline.bem.ai",
                "independent_document_processing_enabled": False,
                "output_schema": {
                    "value": {
                        "type": "object",
                        "required": ["tenders"],
                        "properties": {
                            "tenders": {
                                "type": "array",
                                "items": {
                                    "type": "object",
                                    "required": [
                                        "loadReference",
                                        "origin",
                                        "destination",
                                        "weightTons",
                                        "loadType",
                                        "desiredDeliveryDate",
                                        "bidSubmissionDeadline",
                                        "submitter",
                                    ],
                                    "properties": {
                                        "origin": {
                                            "type": "string",
                                            "description": "The starting point of the shipment.",
                                        },
                                        "loadType": {
                                            "type": "string",
                                            "description": "The type of goods being shipped.",
                                        },
                                        "submitter": {
                                            "type": "object",
                                            "required": ["name", "position", "contactInfo"],
                                            "properties": {
                                                "name": {
                                                    "type": "string",
                                                    "description": "Name of the person submitting the tender.",
                                                },
                                                "position": {
                                                    "type": "string",
                                                    "description": "Position of the submitter within their company.",
                                                },
                                                "contactInfo": {
                                                    "type": "object",
                                                    "required": ["email"],
                                                    "properties": {
                                                        "email": {
                                                            "type": "string",
                                                            "format": "email",
                                                            "description": "Email address of the submitter.",
                                                        },
                                                        "phone": {
                                                            "type": "string",
                                                            "description": "Phone number of the submitter.",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                        "weightTons": {
                                            "type": "number",
                                            "description": "The weight of the load in tons.",
                                        },
                                        "destination": {
                                            "type": "string",
                                            "description": "The endpoint of the shipment.",
                                        },
                                        "loadReference": {
                                            "type": "string",
                                            "description": "Unique identifier for the load tender.",
                                        },
                                        "desiredDeliveryDate": {
                                            "type": "string",
                                            "format": "date",
                                            "description": "The preferred date for the shipment to be delivered.",
                                        },
                                        "bidSubmissionDeadline": {
                                            "type": "string",
                                            "format": "date",
                                            "description": "The deadline for submitting bids.",
                                        },
                                    },
                                },
                            }
                        },
                    }
                },
                "output_schema_name": "Freight Load Schema",
                "next_action_type_config_id": "nextActionTypeConfigID",
            },
            complex_tabular_transform_enabled=False,
            independent_document_processing_enabled=False,
            output_schema_name="Freight Load Schema",
            webhook_enabled=True,
            webhook_url="https://bem-example.ai/test/url",
        )
        assert_matches_type(Pipeline, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.pipelines.with_raw_response.create(
            name="Freight Load Pipeline",
            output_schema={
                "value": {
                    "type": "object",
                    "required": ["tenders"],
                    "properties": {
                        "tenders": {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "required": [
                                    "loadReference",
                                    "origin",
                                    "destination",
                                    "weightTons",
                                    "loadType",
                                    "desiredDeliveryDate",
                                    "bidSubmissionDeadline",
                                    "submitter",
                                ],
                                "properties": {
                                    "origin": {
                                        "type": "string",
                                        "description": "The starting point of the shipment.",
                                    },
                                    "loadType": {
                                        "type": "string",
                                        "description": "The type of goods being shipped.",
                                    },
                                    "submitter": {
                                        "type": "object",
                                        "required": ["name", "position", "contactInfo"],
                                        "properties": {
                                            "name": {
                                                "type": "string",
                                                "description": "Name of the person submitting the tender.",
                                            },
                                            "position": {
                                                "type": "string",
                                                "description": "Position of the submitter within their company.",
                                            },
                                            "contactInfo": {
                                                "type": "object",
                                                "required": ["email"],
                                                "properties": {
                                                    "email": {
                                                        "type": "string",
                                                        "format": "email",
                                                        "description": "Email address of the submitter.",
                                                    },
                                                    "phone": {
                                                        "type": "string",
                                                        "description": "Phone number of the submitter.",
                                                    },
                                                },
                                            },
                                        },
                                    },
                                    "weightTons": {
                                        "type": "number",
                                        "description": "The weight of the load in tons.",
                                    },
                                    "destination": {
                                        "type": "string",
                                        "description": "The endpoint of the shipment.",
                                    },
                                    "loadReference": {
                                        "type": "string",
                                        "description": "Unique identifier for the load tender.",
                                    },
                                    "desiredDeliveryDate": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The preferred date for the shipment to be delivered.",
                                    },
                                    "bidSubmissionDeadline": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The deadline for submitting bids.",
                                    },
                                },
                            },
                        }
                    },
                }
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pipeline = await response.parse()
        assert_matches_type(Pipeline, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncBemSDK) -> None:
        async with async_client.pipelines.with_streaming_response.create(
            name="Freight Load Pipeline",
            output_schema={
                "value": {
                    "type": "object",
                    "required": ["tenders"],
                    "properties": {
                        "tenders": {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "required": [
                                    "loadReference",
                                    "origin",
                                    "destination",
                                    "weightTons",
                                    "loadType",
                                    "desiredDeliveryDate",
                                    "bidSubmissionDeadline",
                                    "submitter",
                                ],
                                "properties": {
                                    "origin": {
                                        "type": "string",
                                        "description": "The starting point of the shipment.",
                                    },
                                    "loadType": {
                                        "type": "string",
                                        "description": "The type of goods being shipped.",
                                    },
                                    "submitter": {
                                        "type": "object",
                                        "required": ["name", "position", "contactInfo"],
                                        "properties": {
                                            "name": {
                                                "type": "string",
                                                "description": "Name of the person submitting the tender.",
                                            },
                                            "position": {
                                                "type": "string",
                                                "description": "Position of the submitter within their company.",
                                            },
                                            "contactInfo": {
                                                "type": "object",
                                                "required": ["email"],
                                                "properties": {
                                                    "email": {
                                                        "type": "string",
                                                        "format": "email",
                                                        "description": "Email address of the submitter.",
                                                    },
                                                    "phone": {
                                                        "type": "string",
                                                        "description": "Phone number of the submitter.",
                                                    },
                                                },
                                            },
                                        },
                                    },
                                    "weightTons": {
                                        "type": "number",
                                        "description": "The weight of the load in tons.",
                                    },
                                    "destination": {
                                        "type": "string",
                                        "description": "The endpoint of the shipment.",
                                    },
                                    "loadReference": {
                                        "type": "string",
                                        "description": "Unique identifier for the load tender.",
                                    },
                                    "desiredDeliveryDate": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The preferred date for the shipment to be delivered.",
                                    },
                                    "bidSubmissionDeadline": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The deadline for submitting bids.",
                                    },
                                },
                            },
                        }
                    },
                }
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pipeline = await response.parse()
            assert_matches_type(Pipeline, pipeline, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncBemSDK) -> None:
        pipeline = await async_client.pipelines.retrieve(
            "pipelineID",
        )
        assert_matches_type(PipelineRetrieveResponse, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.pipelines.with_raw_response.retrieve(
            "pipelineID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pipeline = await response.parse()
        assert_matches_type(PipelineRetrieveResponse, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncBemSDK) -> None:
        async with async_client.pipelines.with_streaming_response.retrieve(
            "pipelineID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pipeline = await response.parse()
            assert_matches_type(PipelineRetrieveResponse, pipeline, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncBemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pipeline_id` but received ''"):
            await async_client.pipelines.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncBemSDK) -> None:
        pipeline = await async_client.pipelines.update(
            pipeline_id="pipelineID",
        )
        assert_matches_type(Pipeline, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncBemSDK) -> None:
        pipeline = await async_client.pipelines.update(
            pipeline_id="pipelineID",
            action_config={
                "action_type_config_id": "actionTypeConfigID",
                "name": "name",
                "action_type": "transform",
                "complex_tabular_transform_enabled": True,
                "email_address": "eml_2c9AXFXHwiaL4vPXDTOS171OJ8T@pipeline.bem.ai",
                "independent_document_processing_enabled": False,
                "output_schema": {
                    "value": {
                        "type": "object",
                        "required": ["tenders"],
                        "properties": {
                            "tenders": {
                                "type": "array",
                                "items": {
                                    "type": "object",
                                    "required": [
                                        "loadReference",
                                        "origin",
                                        "destination",
                                        "weightTons",
                                        "loadType",
                                        "desiredDeliveryDate",
                                        "bidSubmissionDeadline",
                                        "submitter",
                                    ],
                                    "properties": {
                                        "origin": {
                                            "type": "string",
                                            "description": "The starting point of the shipment.",
                                        },
                                        "loadType": {
                                            "type": "string",
                                            "description": "The type of goods being shipped.",
                                        },
                                        "submitter": {
                                            "type": "object",
                                            "required": ["name", "position", "contactInfo"],
                                            "properties": {
                                                "name": {
                                                    "type": "string",
                                                    "description": "Name of the person submitting the tender.",
                                                },
                                                "position": {
                                                    "type": "string",
                                                    "description": "Position of the submitter within their company.",
                                                },
                                                "contactInfo": {
                                                    "type": "object",
                                                    "required": ["email"],
                                                    "properties": {
                                                        "email": {
                                                            "type": "string",
                                                            "format": "email",
                                                            "description": "Email address of the submitter.",
                                                        },
                                                        "phone": {
                                                            "type": "string",
                                                            "description": "Phone number of the submitter.",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                        "weightTons": {
                                            "type": "number",
                                            "description": "The weight of the load in tons.",
                                        },
                                        "destination": {
                                            "type": "string",
                                            "description": "The endpoint of the shipment.",
                                        },
                                        "loadReference": {
                                            "type": "string",
                                            "description": "Unique identifier for the load tender.",
                                        },
                                        "desiredDeliveryDate": {
                                            "type": "string",
                                            "format": "date",
                                            "description": "The preferred date for the shipment to be delivered.",
                                        },
                                        "bidSubmissionDeadline": {
                                            "type": "string",
                                            "format": "date",
                                            "description": "The deadline for submitting bids.",
                                        },
                                    },
                                },
                            }
                        },
                    }
                },
                "output_schema_name": "Freight Load Schema",
                "next_action_type_config_id": "nextActionTypeConfigID",
            },
            complex_tabular_transform_enabled=False,
            independent_document_processing_enabled=False,
            name="Freight invoices pipeline",
            output_schema={
                "value": {
                    "type": "object",
                    "required": ["tenders"],
                    "properties": {
                        "tenders": {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "required": [
                                    "loadReference",
                                    "origin",
                                    "destination",
                                    "weightTons",
                                    "loadType",
                                    "desiredDeliveryDate",
                                    "bidSubmissionDeadline",
                                    "submitter",
                                ],
                                "properties": {
                                    "origin": {
                                        "type": "string",
                                        "description": "The starting point of the shipment.",
                                    },
                                    "loadType": {
                                        "type": "string",
                                        "description": "The type of goods being shipped.",
                                    },
                                    "submitter": {
                                        "type": "object",
                                        "required": ["name", "position", "contactInfo"],
                                        "properties": {
                                            "name": {
                                                "type": "string",
                                                "description": "Name of the person submitting the tender.",
                                            },
                                            "position": {
                                                "type": "string",
                                                "description": "Position of the submitter within their company.",
                                            },
                                            "contactInfo": {
                                                "type": "object",
                                                "required": ["email"],
                                                "properties": {
                                                    "email": {
                                                        "type": "string",
                                                        "format": "email",
                                                        "description": "Email address of the submitter.",
                                                    },
                                                    "phone": {
                                                        "type": "string",
                                                        "description": "Phone number of the submitter.",
                                                    },
                                                },
                                            },
                                        },
                                    },
                                    "weightTons": {
                                        "type": "number",
                                        "description": "The weight of the load in tons.",
                                    },
                                    "destination": {
                                        "type": "string",
                                        "description": "The endpoint of the shipment.",
                                    },
                                    "loadReference": {
                                        "type": "string",
                                        "description": "Unique identifier for the load tender.",
                                    },
                                    "desiredDeliveryDate": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The preferred date for the shipment to be delivered.",
                                    },
                                    "bidSubmissionDeadline": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The deadline for submitting bids.",
                                    },
                                },
                            },
                        }
                    },
                }
            },
            output_schema_name="Freight invoices schema",
            webhook_enabled=True,
            webhook_url="https://bem-example.ai/test/url",
        )
        assert_matches_type(Pipeline, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.pipelines.with_raw_response.update(
            pipeline_id="pipelineID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pipeline = await response.parse()
        assert_matches_type(Pipeline, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncBemSDK) -> None:
        async with async_client.pipelines.with_streaming_response.update(
            pipeline_id="pipelineID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pipeline = await response.parse()
            assert_matches_type(Pipeline, pipeline, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update(self, async_client: AsyncBemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pipeline_id` but received ''"):
            await async_client.pipelines.with_raw_response.update(
                pipeline_id="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncBemSDK) -> None:
        pipeline = await async_client.pipelines.list()
        assert_matches_type(PipelineListResponse, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncBemSDK) -> None:
        pipeline = await async_client.pipelines.list(
            ending_before="endingBefore",
            limit=1,
            starting_after="startingAfter",
        )
        assert_matches_type(PipelineListResponse, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.pipelines.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pipeline = await response.parse()
        assert_matches_type(PipelineListResponse, pipeline, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncBemSDK) -> None:
        async with async_client.pipelines.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pipeline = await response.parse()
            assert_matches_type(PipelineListResponse, pipeline, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncBemSDK) -> None:
        pipeline = await async_client.pipelines.delete(
            "pipelineID",
        )
        assert pipeline is None

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.pipelines.with_raw_response.delete(
            "pipelineID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pipeline = await response.parse()
        assert pipeline is None

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncBemSDK) -> None:
        async with async_client.pipelines.with_streaming_response.delete(
            "pipelineID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pipeline = await response.parse()
            assert pipeline is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncBemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pipeline_id` but received ''"):
            await async_client.pipelines.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tasks import (
    TasksResource,
    AsyncTasksResource,
    TasksResourceWithRawResponse,
    AsyncTasksResourceWithRawResponse,
    TasksResourceWithStreamingResponse,
    AsyncTasksResourceWithStreamingResponse,
)
from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .pipelines import (
    PipelinesResource,
    AsyncPipelinesResource,
    PipelinesResourceWithRawResponse,
    AsyncPipelinesResourceWithRawResponse,
    PipelinesResourceWithStreamingResponse,
    AsyncPipelinesResourceWithStreamingResponse,
)
from .subscriptions import (
    SubscriptionsResource,
    AsyncSubscriptionsResource,
    SubscriptionsResourceWithRawResponse,
    AsyncSubscriptionsResourceWithRawResponse,
    SubscriptionsResourceWithStreamingResponse,
    AsyncSubscriptionsResourceWithStreamingResponse,
)
from .webhook_secret import (
    WebhookSecretResource,
    AsyncWebhookSecretResource,
    WebhookSecretResourceWithRawResponse,
    AsyncWebhookSecretResourceWithRawResponse,
    WebhookSecretResourceWithStreamingResponse,
    AsyncWebhookSecretResourceWithStreamingResponse,
)
from .transformations import (
    TransformationsResource,
    AsyncTransformationsResource,
    TransformationsResourceWithRawResponse,
    AsyncTransformationsResourceWithRawResponse,
    TransformationsResourceWithStreamingResponse,
    AsyncTransformationsResourceWithStreamingResponse,
)
from .action_type_configs import (
    ActionTypeConfigsResource,
    AsyncActionTypeConfigsResource,
    ActionTypeConfigsResourceWithRawResponse,
    AsyncActionTypeConfigsResourceWithRawResponse,
    ActionTypeConfigsResourceWithStreamingResponse,
    AsyncActionTypeConfigsResourceWithStreamingResponse,
)

__all__ = [
    "ActionTypeConfigsResource",
    "AsyncActionTypeConfigsResource",
    "ActionTypeConfigsResourceWithRawResponse",
    "AsyncActionTypeConfigsResourceWithRawResponse",
    "ActionTypeConfigsResourceWithStreamingResponse",
    "AsyncActionTypeConfigsResourceWithStreamingResponse",
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "EventsResource",
    "AsyncEventsResource",
    "EventsResourceWithRawResponse",
    "AsyncEventsResourceWithRawResponse",
    "EventsResourceWithStreamingResponse",
    "AsyncEventsResourceWithStreamingResponse",
    "TasksResource",
    "AsyncTasksResource",
    "TasksResourceWithRawResponse",
    "AsyncTasksResourceWithRawResponse",
    "TasksResourceWithStreamingResponse",
    "AsyncTasksResourceWithStreamingResponse",
    "SubscriptionsResource",
    "AsyncSubscriptionsResource",
    "SubscriptionsResourceWithRawResponse",
    "AsyncSubscriptionsResourceWithRawResponse",
    "SubscriptionsResourceWithStreamingResponse",
    "AsyncSubscriptionsResourceWithStreamingResponse",
    "PipelinesResource",
    "AsyncPipelinesResource",
    "PipelinesResourceWithRawResponse",
    "AsyncPipelinesResourceWithRawResponse",
    "PipelinesResourceWithStreamingResponse",
    "AsyncPipelinesResourceWithStreamingResponse",
    "TransformationsResource",
    "AsyncTransformationsResource",
    "TransformationsResourceWithRawResponse",
    "AsyncTransformationsResourceWithRawResponse",
    "TransformationsResourceWithStreamingResponse",
    "AsyncTransformationsResourceWithStreamingResponse",
    "WebhookSecretResource",
    "AsyncWebhookSecretResource",
    "WebhookSecretResourceWithRawResponse",
    "AsyncWebhookSecretResourceWithRawResponse",
    "WebhookSecretResourceWithStreamingResponse",
    "AsyncWebhookSecretResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ActionTypeConfigBaseParam"]


class ActionTypeConfigBaseParam(TypedDict, total=False):
    action_type_config_id: Required[Annotated[str, PropertyInfo(alias="actionTypeConfigID")]]
    """Unique identifier of action type config."""

    name: Required[str]
    """Name of action type config."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["ActionType"]

ActionType: TypeAlias = Literal["transform", "route", "split", "join", "email"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["TransformationListErrorsParams"]


class TransformationListErrorsParams(TypedDict, total=False):
    reference_ids: Required[Annotated[List[str], PropertyInfo(alias="referenceIDs")]]
    """Array with the reference IDs of the transformed data points.

    Formatted as a CSV array.
    """

    ending_before: Annotated[str, PropertyInfo(alias="endingBefore")]
    """A cursor to use in pagination.

    `endingBefore` is a transform ID that defines your place in the list. For
    example, if you make a list request and receive 50 objects, starting with
    `tr_2bxoJPNdSD4LgRT4YVC4gt72hlI`, your subsequent call can include
    `endingBefore=tr_2bxoJPNdSD4LgRT4YVC4gt72hlI` to fetch the previous page of the
    list.
    """

    limit: int
    """
    This specifies a limit on the number of objects to return, ranging between 1
    and 100.
    """

    pipeline_id: Annotated[str, PropertyInfo(alias="pipelineID")]
    """The unique ID for a given pipeline.

    Will filter to just the transformations processed by the given pipeline. If left
    out, will query over ALL transformations for your account.
    """

    sort_order: Annotated[Literal["asc", "desc"], PropertyInfo(alias="sortOrder")]
    """Specifies sorting behavior.

    The two options are `asc` and `desc` to sort ascending and descending
    respectively, with default sort being ascending. Paging works in both
    directions.
    """

    starting_after: Annotated[str, PropertyInfo(alias="startingAfter")]
    """A cursor to use in pagination.

    `startingAfter` is a transform ID that defines your place in the list. For
    example, if you make a list request and receive 50 objects, ending with
    `tr_2bxoJPNdSD4LgRT4YVC4gt72hlI`, your subsequent call can include
    `startingAfter=tr_2bxoJPNdSD4LgRT4YVC4gt72hlI` to fetch the next page of the
    list.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from bem import BemSDK, AsyncBemSDK
from bem.types import (
    GetActionsResponse,
)
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestActions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_overload_1(self, client: BemSDK) -> None:
        action = client.actions.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create_overload_1(self, client: BemSDK) -> None:
        response = client.actions.with_raw_response.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create_overload_1(self, client: BemSDK) -> None:
        with client.actions.with_streaming_response.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert_matches_type(GetActionsResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_create_overload_2(self, client: BemSDK) -> None:
        action = client.actions.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create_overload_2(self, client: BemSDK) -> None:
        response = client.actions.with_raw_response.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create_overload_2(self, client: BemSDK) -> None:
        with client.actions.with_streaming_response.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert_matches_type(GetActionsResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_create_overload_3(self, client: BemSDK) -> None:
        action = client.actions.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create_overload_3(self, client: BemSDK) -> None:
        response = client.actions.with_raw_response.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create_overload_3(self, client: BemSDK) -> None:
        with client.actions.with_streaming_response.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert_matches_type(GetActionsResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_create_overload_4(self, client: BemSDK) -> None:
        action = client.actions.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "inputs": [
                        {
                            "input_content": "U3RhaW5sZXNzIHJvY2tz",
                            "input_type": "email",
                        }
                    ],
                    "join_type": "standard",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create_overload_4(self, client: BemSDK) -> None:
        response = client.actions.with_raw_response.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "inputs": [
                        {
                            "input_content": "U3RhaW5sZXNzIHJvY2tz",
                            "input_type": "email",
                        }
                    ],
                    "join_type": "standard",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create_overload_4(self, client: BemSDK) -> None:
        with client.actions.with_streaming_response.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "inputs": [
                        {
                            "input_content": "U3RhaW5sZXNzIHJvY2tz",
                            "input_type": "email",
                        }
                    ],
                    "join_type": "standard",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert_matches_type(GetActionsResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_create_overload_5(self, client: BemSDK) -> None:
        action = client.actions.create(
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params_overload_5(self, client: BemSDK) -> None:
        action = client.actions.create(
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
            actions=[
                {
                    "reference_id": "referenceID",
                    "template_variables": {},
                }
            ],
        )
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create_overload_5(self, client: BemSDK) -> None:
        response = client.actions.with_raw_response.create(
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create_overload_5(self, client: BemSDK) -> None:
        with client.actions.with_streaming_response.create(
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert_matches_type(GetActionsResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: BemSDK) -> None:
        action = client.actions.list()
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_with_all_params(self, client: BemSDK) -> None:
        action = client.actions.list(
            action_ids=["string"],
            action_type="transform",
            action_type_config_ids=["string"],
            ending_before="endingBefore",
            limit=1,
            sort_order="asc",
            starting_after="startingAfter",
        )
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: BemSDK) -> None:
        response = client.actions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: BemSDK) -> None:
        with client.actions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert_matches_type(GetActionsResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_correct_route(self, client: BemSDK) -> None:
        action = client.actions.correct_route()
        assert action is None

    @pytest.mark.skip()
    @parametrize
    def test_method_correct_route_with_all_params(self, client: BemSDK) -> None:
        action = client.actions.correct_route(
            route_actions=[
                {
                    "action_id": "actionID",
                    "corrected_choice": "correctedChoice",
                }
            ],
        )
        assert action is None

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_correct_route(self, client: BemSDK) -> None:
        response = client.actions.with_raw_response.correct_route()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert action is None

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_correct_route(self, client: BemSDK) -> None:
        with client.actions.with_streaming_response.correct_route() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert action is None

        assert cast(Any, response.is_closed) is True


class TestAsyncActions:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_overload_1(self, async_client: AsyncBemSDK) -> None:
        action = await async_client.actions.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create_overload_1(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.actions.with_raw_response.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create_overload_1(self, async_client: AsyncBemSDK) -> None:
        async with async_client.actions.with_streaming_response.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert_matches_type(GetActionsResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_overload_2(self, async_client: AsyncBemSDK) -> None:
        action = await async_client.actions.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create_overload_2(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.actions.with_raw_response.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create_overload_2(self, async_client: AsyncBemSDK) -> None:
        async with async_client.actions.with_streaming_response.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert_matches_type(GetActionsResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_overload_3(self, async_client: AsyncBemSDK) -> None:
        action = await async_client.actions.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create_overload_3(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.actions.with_raw_response.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create_overload_3(self, async_client: AsyncBemSDK) -> None:
        async with async_client.actions.with_streaming_response.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert_matches_type(GetActionsResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_overload_4(self, async_client: AsyncBemSDK) -> None:
        action = await async_client.actions.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "inputs": [
                        {
                            "input_content": "U3RhaW5sZXNzIHJvY2tz",
                            "input_type": "email",
                        }
                    ],
                    "join_type": "standard",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create_overload_4(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.actions.with_raw_response.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "inputs": [
                        {
                            "input_content": "U3RhaW5sZXNzIHJvY2tz",
                            "input_type": "email",
                        }
                    ],
                    "join_type": "standard",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create_overload_4(self, async_client: AsyncBemSDK) -> None:
        async with async_client.actions.with_streaming_response.create(
            actions=[
                {
                    "reference_id": "referenceID",
                    "inputs": [
                        {
                            "input_content": "U3RhaW5sZXNzIHJvY2tz",
                            "input_type": "email",
                        }
                    ],
                    "join_type": "standard",
                }
            ],
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert_matches_type(GetActionsResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_overload_5(self, async_client: AsyncBemSDK) -> None:
        action = await async_client.actions.create(
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params_overload_5(self, async_client: AsyncBemSDK) -> None:
        action = await async_client.actions.create(
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
            actions=[
                {
                    "reference_id": "referenceID",
                    "template_variables": {},
                }
            ],
        )
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create_overload_5(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.actions.with_raw_response.create(
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create_overload_5(self, async_client: AsyncBemSDK) -> None:
        async with async_client.actions.with_streaming_response.create(
            action_type="transform",
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert_matches_type(GetActionsResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncBemSDK) -> None:
        action = await async_client.actions.list()
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncBemSDK) -> None:
        action = await async_client.actions.list(
            action_ids=["string"],
            action_type="transform",
            action_type_config_ids=["string"],
            ending_before="endingBefore",
            limit=1,
            sort_order="asc",
            starting_after="startingAfter",
        )
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.actions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert_matches_type(GetActionsResponse, action, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncBemSDK) -> None:
        async with async_client.actions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert_matches_type(GetActionsResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_correct_route(self, async_client: AsyncBemSDK) -> None:
        action = await async_client.actions.correct_route()
        assert action is None

    @pytest.mark.skip()
    @parametrize
    async def test_method_correct_route_with_all_params(self, async_client: AsyncBemSDK) -> None:
        action = await async_client.actions.correct_route(
            route_actions=[
                {
                    "action_id": "actionID",
                    "corrected_choice": "correctedChoice",
                }
            ],
        )
        assert action is None

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_correct_route(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.actions.with_raw_response.correct_route()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert action is None

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_correct_route(self, async_client: AsyncBemSDK) -> None:
        async with async_client.actions.with_streaming_response.correct_route() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert action is None

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .pipeline import Pipeline

__all__ = ["PipelineRetrieveResponse"]


class PipelineRetrieveResponse(BaseModel):
    pipeline: Optional[Pipeline] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .action_base import ActionBase
from .join_action_base import JoinActionBase
from .email_action_base import EmailActionBase
from .route_action_base import RouteActionBase
from .split_action_base import SplitActionBase
from .transform_action_base import TransformActionBase

__all__ = [
    "GetActionsResponse",
    "ActionTransformAction",
    "ActionRouteAction",
    "ActionSplitAction",
    "ActionJoinAction",
    "ActionEmailAction",
]


class ActionTransformAction(ActionBase, TransformActionBase):
    action_type: Literal["transform"] = FieldInfo(alias="actionType")


class ActionRouteAction(ActionBase, RouteActionBase):
    action_type: Literal["route"] = FieldInfo(alias="actionType")


class ActionSplitAction(ActionBase, SplitActionBase):
    action_type: Literal["split"] = FieldInfo(alias="actionType")


class ActionJoinAction(ActionBase, JoinActionBase):
    action_type: Literal["join"] = FieldInfo(alias="actionType")


class ActionEmailAction(ActionBase, EmailActionBase):
    action_type: Literal["email"] = FieldInfo(alias="actionType")


class GetActionsResponse(BaseModel):
    actions: Optional[
        List[Union[ActionTransformAction, ActionRouteAction, ActionSplitAction, ActionJoinAction, ActionEmailAction]]
    ] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from bem import BemSDK, AsyncBemSDK
from bem.types import WebhookSecret
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestWebhookSecret:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: BemSDK) -> None:
        webhook_secret = client.webhook_secret.create()
        assert_matches_type(WebhookSecret, webhook_secret, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: BemSDK) -> None:
        response = client.webhook_secret.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook_secret = response.parse()
        assert_matches_type(WebhookSecret, webhook_secret, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: BemSDK) -> None:
        with client.webhook_secret.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook_secret = response.parse()
            assert_matches_type(WebhookSecret, webhook_secret, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: BemSDK) -> None:
        webhook_secret = client.webhook_secret.retrieve()
        assert_matches_type(WebhookSecret, webhook_secret, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: BemSDK) -> None:
        response = client.webhook_secret.with_raw_response.retrieve()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook_secret = response.parse()
        assert_matches_type(WebhookSecret, webhook_secret, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: BemSDK) -> None:
        with client.webhook_secret.with_streaming_response.retrieve() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook_secret = response.parse()
            assert_matches_type(WebhookSecret, webhook_secret, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncWebhookSecret:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncBemSDK) -> None:
        webhook_secret = await async_client.webhook_secret.create()
        assert_matches_type(WebhookSecret, webhook_secret, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.webhook_secret.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook_secret = await response.parse()
        assert_matches_type(WebhookSecret, webhook_secret, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncBemSDK) -> None:
        async with async_client.webhook_secret.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook_secret = await response.parse()
            assert_matches_type(WebhookSecret, webhook_secret, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncBemSDK) -> None:
        webhook_secret = await async_client.webhook_secret.retrieve()
        assert_matches_type(WebhookSecret, webhook_secret, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.webhook_secret.with_raw_response.retrieve()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook_secret = await response.parse()
        assert_matches_type(WebhookSecret, webhook_secret, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncBemSDK) -> None:
        async with async_client.webhook_secret.with_streaming_response.retrieve() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook_secret = await response.parse()
            assert_matches_type(WebhookSecret, webhook_secret, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .event import Event
from .._models import BaseModel

__all__ = ["EventListResponse"]


class EventListResponse(BaseModel):
    events: Optional[List[Event]] = None

    total_count: Optional[int] = FieldInfo(alias="totalCount", default=None)
    """The total number of results available."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Mapping, Iterable, cast
from typing_extensions import Literal, overload

import httpx

from ..types import ActionType, action_list_params, action_create_params, action_correct_route_params
from .._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven
from .._utils import extract_files, required_args, maybe_transform, deepcopy_minimal, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.action_type import ActionType
from ..types.get_actions_response import GetActionsResponse

__all__ = ["ActionsResource", "AsyncActionsResource"]


class ActionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#accessing-raw-response-data-eg-headers
        """
        return ActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#with_streaming_response
        """
        return ActionsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        actions: Iterable[action_create_params.CreateTransformActionsAction],
        action_type: ActionType,
        action_type_config_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GetActionsResponse:
        """
        Create a batch of Actions

        Args:
          action_type: The type of the action.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        actions: Iterable[action_create_params.CreateRouteActionsAction],
        action_type: ActionType,
        action_type_config_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GetActionsResponse:
        """
        Create a batch of Actions

        Args:
          action_type: The type of the action.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        actions: Iterable[action_create_params.CreateSplitActionsAction],
        action_type: ActionType,
        action_type_config_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GetActionsResponse:
        """
        Create a batch of Actions

        Args:
          action_type: The type of the action.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        actions: Iterable[action_create_params.CreateJoinActionsAction],
        action_type: ActionType,
        action_type_config_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GetActionsResponse:
        """
        Create a batch of Actions

        Args:
          action_type: The type of the action.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        action_type: ActionType,
        action_type_config_id: str,
        actions: Iterable[action_create_params.CreateEmailActionsAction] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GetActionsResponse:
        """
        Create a batch of Actions

        Args:
          action_type: The type of the action.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["actions", "action_type", "action_type_config_id"], ["action_type", "action_type_config_id"])
    def create(
        self,
        *,
        actions: Iterable[action_create_params.CreateTransformActionsAction]
        | Iterable[action_create_params.CreateRouteActionsAction]
        | Iterable[action_create_params.CreateSplitActionsAction]
        | Iterable[action_create_params.CreateJoinActionsAction]
        | Iterable[action_create_params.CreateEmailActionsAction]
        | NotGiven = NOT_GIVEN,
        action_type: ActionType,
        action_type_config_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GetActionsResponse:
        body = deepcopy_minimal(
            {
                "actions": actions,
                "action_type": action_type,
                "action_type_config_id": action_type_config_id,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"], ["files", "<array>"]])
        if files:
            # It should be noted that the actual Content-Type header that will be
            # sent to the server will contain a `boundary` parameter, e.g.
            # multipart/form-data; boundary=---abc--
            extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/v1-alpha/actions",
            body=maybe_transform(body, action_create_params.ActionCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GetActionsResponse,
        )

    def list(
        self,
        *,
        action_ids: List[str] | NotGiven = NOT_GIVEN,
        action_type: ActionType | NotGiven = NOT_GIVEN,
        action_type_config_ids: List[str] | NotGiven = NOT_GIVEN,
        ending_before: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        sort_order: Literal["asc", "desc"] | NotGiven = NOT_GIVEN,
        starting_after: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GetActionsResponse:
        """
        List Actions

        Args:
          action_type: The type of the action.

          ending_before: A cursor to use in pagination. `endingBefore` is an action ID that defines your
              place in the list.

          sort_order: Specifies sorting behavior. The two options are `asc` and `desc` to sort
              ascending and descending respectively, with default sort being ascending. Paging
              works in both directions.

          starting_after: A cursor to use in pagination. `startingAfter` is an action ID that defines your
              place in the list.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1-alpha/actions",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "action_ids": action_ids,
                        "action_type": action_type,
                        "action_type_config_ids": action_type_config_ids,
                        "ending_before": ending_before,
                        "limit": limit,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                    },
                    action_list_params.ActionListParams,
                ),
            ),
            cast_to=GetActionsResponse,
        )

    def correct_route(
        self,
        *,
        route_actions: Iterable[action_correct_route_params.RouteAction] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Updates a route event with feedback on the desired router choices.

        Args:
          route_actions: An array of objects containing all the route actions you want to submit feedback
              for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._patch(
            "/v1-alpha/actions/route",
            body=maybe_transform(
                {"route_actions": route_actions}, action_correct_route_params.ActionCorrectRouteParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncActionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#with_streaming_response
        """
        return AsyncActionsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        actions: Iterable[action_create_params.CreateTransformActionsAction],
        action_type: ActionType,
        action_type_config_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GetActionsResponse:
        """
        Create a batch of Actions

        Args:
          action_type: The type of the action.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        actions: Iterable[action_create_params.CreateRouteActionsAction],
        action_type: ActionType,
        action_type_config_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GetActionsResponse:
        """
        Create a batch of Actions

        Args:
          action_type: The type of the action.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        actions: Iterable[action_create_params.CreateSplitActionsAction],
        action_type: ActionType,
        action_type_config_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GetActionsResponse:
        """
        Create a batch of Actions

        Args:
          action_type: The type of the action.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        actions: Iterable[action_create_params.CreateJoinActionsAction],
        action_type: ActionType,
        action_type_config_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GetActionsResponse:
        """
        Create a batch of Actions

        Args:
          action_type: The type of the action.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        action_type: ActionType,
        action_type_config_id: str,
        actions: Iterable[action_create_params.CreateEmailActionsAction] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GetActionsResponse:
        """
        Create a batch of Actions

        Args:
          action_type: The type of the action.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["actions", "action_type", "action_type_config_id"], ["action_type", "action_type_config_id"])
    async def create(
        self,
        *,
        actions: Iterable[action_create_params.CreateTransformActionsAction]
        | Iterable[action_create_params.CreateRouteActionsAction]
        | Iterable[action_create_params.CreateSplitActionsAction]
        | Iterable[action_create_params.CreateJoinActionsAction]
        | Iterable[action_create_params.CreateEmailActionsAction]
        | NotGiven = NOT_GIVEN,
        action_type: ActionType,
        action_type_config_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GetActionsResponse:
        body = deepcopy_minimal(
            {
                "actions": actions,
                "action_type": action_type,
                "action_type_config_id": action_type_config_id,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"], ["files", "<array>"]])
        if files:
            # It should be noted that the actual Content-Type header that will be
            # sent to the server will contain a `boundary` parameter, e.g.
            # multipart/form-data; boundary=---abc--
            extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/v1-alpha/actions",
            body=await async_maybe_transform(body, action_create_params.ActionCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GetActionsResponse,
        )

    async def list(
        self,
        *,
        action_ids: List[str] | NotGiven = NOT_GIVEN,
        action_type: ActionType | NotGiven = NOT_GIVEN,
        action_type_config_ids: List[str] | NotGiven = NOT_GIVEN,
        ending_before: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        sort_order: Literal["asc", "desc"] | NotGiven = NOT_GIVEN,
        starting_after: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GetActionsResponse:
        """
        List Actions

        Args:
          action_type: The type of the action.

          ending_before: A cursor to use in pagination. `endingBefore` is an action ID that defines your
              place in the list.

          sort_order: Specifies sorting behavior. The two options are `asc` and `desc` to sort
              ascending and descending respectively, with default sort being ascending. Paging
              works in both directions.

          starting_after: A cursor to use in pagination. `startingAfter` is an action ID that defines your
              place in the list.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1-alpha/actions",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "action_ids": action_ids,
                        "action_type": action_type,
                        "action_type_config_ids": action_type_config_ids,
                        "ending_before": ending_before,
                        "limit": limit,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                    },
                    action_list_params.ActionListParams,
                ),
            ),
            cast_to=GetActionsResponse,
        )

    async def correct_route(
        self,
        *,
        route_actions: Iterable[action_correct_route_params.RouteAction] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Updates a route event with feedback on the desired router choices.

        Args:
          route_actions: An array of objects containing all the route actions you want to submit feedback
              for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._patch(
            "/v1-alpha/actions/route",
            body=await async_maybe_transform(
                {"route_actions": route_actions}, action_correct_route_params.ActionCorrectRouteParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class ActionsResourceWithRawResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.create = to_raw_response_wrapper(
            actions.create,
        )
        self.list = to_raw_response_wrapper(
            actions.list,
        )
        self.correct_route = to_raw_response_wrapper(
            actions.correct_route,
        )


class AsyncActionsResourceWithRawResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.create = async_to_raw_response_wrapper(
            actions.create,
        )
        self.list = async_to_raw_response_wrapper(
            actions.list,
        )
        self.correct_route = async_to_raw_response_wrapper(
            actions.correct_route,
        )


class ActionsResourceWithStreamingResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.create = to_streamed_response_wrapper(
            actions.create,
        )
        self.list = to_streamed_response_wrapper(
            actions.list,
        )
        self.correct_route = to_streamed_response_wrapper(
            actions.correct_route,
        )


class AsyncActionsResourceWithStreamingResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.create = async_to_streamed_response_wrapper(
            actions.create,
        )
        self.list = async_to_streamed_response_wrapper(
            actions.list,
        )
        self.correct_route = async_to_streamed_response_wrapper(
            actions.correct_route,
        )
